unit WTSUtils;

interface

uses Windows;

const
      WTS_CURRENT_SERVER_HANDLE = THandle(0);
      WTSAPI = 'wtsapi32.dll';
      NOTIFY_FOR_ALL_SESSIONS = 1;

type      
  WTS_CONNECTSTATE_CLASS = (
    WTSActive,
    WTSConnected,
    WTSConnectQuery,
    WTSShadow,
    WTSDisconnected,
    WTSIdle,
    WTSListen,
    WTSReset,
    WTSDown,
    WTSInit);

  WTS_INFO_CLASS = (
    WTSInitialProgram,
    WTSApplicationName,
    WTSWorkingDirectory,
    WTSOEMId,
    WTSSessionId,
    WTSUserName,
    WTSWinStationName,
    WTSDomainName,
    WTSConnectState,
    WTSClientBuildNumber,
    WTSClientName,
    WTSClientDirectory,
    WTSClientProductId,
    WTSClientHardwareId,
    WTSClientAddress,
    WTSClientDisplay,
    WTSClientProtocolType);

  PWTS_SESSION_INFO = ^WTS_SESSION_INFO;
  WTS_SESSION_INFO = record
    SessionId: DWORD;
    pWinStationName: LPSTR;
    State: WTS_CONNECTSTATE_CLASS;
  end;

function WTSQuerySessionInformation(hServer: cardinal; SessionId: DWORD; WTSInfoClass: WTS_INFO_CLASS; var ppBuffer: Pointer; var pBytesReturned: DWORD): BOOL; stdcall; external wtsapi name 'WTSQuerySessionInformationA'
function WTSSendMessage(hServer: cardinal; SessionId: DWORD; pTitle: LPSTR; TitleLength: DWORD; pMessage: LPSTR; MessageLength: DWORD; Style: DWORD; Timeout: DWORD; var pResponse: DWORD; bWait: BOOL): BOOL; stdcall; external wtsapi name 'WTSSendMessageA';
function WTSEnumerateSessions(hServer: cardinal; Reserved: DWORD; Version: DWORD; var ppSessionInfo: PWTS_SESSION_INFO; var pCount: DWORD): BOOL; stdcall; external wtsapi name 'WTSEnumerateSessionsA';
procedure WTSFreeMemory(pMemory: Pointer); stdcall; external wtsapi name 'WTSFreeMemory';

function RegisterSessionNotification(Wnd: HWND; dwFlags: DWORD): Boolean;
function UnRegisterSessionNotification(Wnd: HWND): Boolean;
procedure NotifierSession(SessionID: cardinal ; _Title, _Message: string ; WaitSeconds: cardinal=5 ; AlwaysSend: bool=false);

var ListeSessionsID: array of cardinal;

implementation

// Active la notification dans la fentre Wnd des informations de changement de session
function RegisterSessionNotification(Wnd: HWND; dwFlags: DWORD): Boolean;
type
  TWTSRegisterSessionNotification = function(Wnd: HWND; dwFlags: DWORD): BOOL; stdcall;
var
  hWTSapi32dll: THandle;
  WTSRegisterSessionNotification: TWTSRegisterSessionNotification;
begin
  Result := False;
  hWTSAPI32DLL := LoadLibrary('Wtsapi32.dll');
  if (hWTSAPI32DLL > 0) then
  begin
    try @WTSRegisterSessionNotification :=
        GetProcAddress(hWTSAPI32DLL, 'WTSRegisterSessionNotification');
      if Assigned(WTSRegisterSessionNotification) then
      begin
        Result:= WTSRegisterSessionNotification(Wnd, dwFlags);
      end;
    finally
      if hWTSAPI32DLL > 0 then
        FreeLibrary(hWTSAPI32DLL);
    end;
  end;
end;

// Dsctive la notification des informations de changement de session
function UnRegisterSessionNotification(Wnd: HWND): Boolean;
type
  TWTSUnRegisterSessionNotification = function(Wnd: HWND): BOOL; stdcall;
var
  hWTSapi32dll: THandle;
  WTSUnRegisterSessionNotification: TWTSUnRegisterSessionNotification;
begin
  Result := False;
  hWTSAPI32DLL := LoadLibrary('Wtsapi32.dll');
  if (hWTSAPI32DLL > 0) then
  begin
    try @WTSUnRegisterSessionNotification :=
        GetProcAddress(hWTSAPI32DLL, 'WTSUnRegisterSessionNotification');
      if Assigned(WTSUnRegisterSessionNotification) then
      begin
        Result:= WTSUnRegisterSessionNotification(Wnd);
      end;
    finally
      if hWTSAPI32DLL > 0 then
        FreeLibrary(hWTSAPI32DLL);
    end;
  end;
end;

procedure NotifierSession(SessionID: cardinal ; _Title, _Message: string ; WaitSeconds: cardinal=5 ; AlwaysSend: bool=false);
var Response: DWORD;
    i: integer;
    Trouve: bool;
    UserName: Pointer;
    cBytesReturned: DWORD;
begin
     i:=0; Trouve:=AlwaysSend; while not Trouve and (i<length(ListeSessionsID)) do if SessionID=ListeSessionsID[i] then Trouve:=true else Inc(i);
     if (AlwaysSend or not Trouve) and WTSQuerySessionInformation(WTS_CURRENT_SERVER_HANDLE, SessionID, WTSUserName, UserName, cBytesReturned) and (length(string(PChar(UserName)))<>0) then
     begin
          WTSSendMessage(WTS_CURRENT_SERVER_HANDLE, SessionID, PChar(_Title), length(_Title), PChar(_Message), length(_Message), MB_OK or MB_ICONINFORMATION, WaitSeconds, Response, false);
          Setlength(ListeSessionsID, length(ListeSessionsID)+1);
          ListeSessionsID[length(ListeSessionsID)-1]:=SessionID;
     end;
end;

initialization
Setlength(ListeSessionsID, 0);

finalization
Finalize(ListeSessionsID);

end.
 