unit WinCred;

interface

uses Windows;

const CREDAPI = 'advapi32.dll';
      CRED_TYPE_GENERIC                 = 1;
      CRED_TYPE_DOMAIN_PASSWORD         = 2;
      CRED_TYPE_DEFAULT=CRED_TYPE_DOMAIN_PASSWORD;
      CRED_PERSIST_NONE          = 0;
      CRED_PERSIST_SESSION       = 1;
      CRED_PERSIST_LOCAL_MACHINE = 2;
      CRED_PERSIST_ENTERPRISE    = 3;

type LPBYTE = {$IFDEF USE_DELPHI_TYPES} Windows.PBYTE {$ELSE} ^Byte {$ENDIF};

     PCREDENTIAL_ATTRIBUTEW = ^CREDENTIAL_ATTRIBUTEW;
     _CREDENTIAL_ATTRIBUTEW = record
       Keyword: LPWSTR;
       Flags: DWORD;
       ValueSize: DWORD;
       Value: LPBYTE;
     end;
     CREDENTIAL_ATTRIBUTEW = _CREDENTIAL_ATTRIBUTEW;

     PCREDENTIALW = ^CREDENTIALW;
     _CREDENTIALW = record
       Flags: DWORD;
       Type_: DWORD;
       TargetName: LPWSTR;
       Comment: LPWSTR;
       LastWritten: FILETIME;
       CredentialBlobSize: DWORD;
       CredentialBlob: LPBYTE;
       Persist: DWORD;
       AttributeCount: DWORD;
       Attributes: PCREDENTIAL_ATTRIBUTEW;
       TargetAlias: LPWSTR;
       UserName: LPWSTR;
     end;
     CREDENTIALW = _CREDENTIALW;

     PVOID = Pointer;

function CredReadCredentials(const Target: WideString ; var UserName, Password: WideString ; CredType: DWORD=CRED_TYPE_DEFAULT): bool;
function CredWriteCredentials(const Target, Username, Password: WideString ; CredType: DWORD=CRED_TYPE_DEFAULT): Boolean;
function CredDeleteCredentials(const Target: WideString ; CredType: DWORD=CRED_TYPE_DEFAULT): Boolean;
function CredGetMaxPersistType(CredType: DWORD; var MaxCredPersistType: DWORD): Boolean;

implementation

function CredWriteW(Credential: PCREDENTIALW; Flags: DWORD): BOOL; stdcall; external CREDAPI name 'CredWriteW';
function CredGetSessionTypes(MaximumPersistCount: DWORD; MaximumPersist: LPDWORD): BOOL; stdcall; external CREDAPI name 'CredGetSessionTypes';
function CredDeleteW(TargetName: LPCWSTR; Type_: DWORD; Flags: DWORD): BOOL; stdcall; external CREDAPI name 'CredDeleteW';
function CredReadW(TargetName: LPCWSTR; Type_: DWORD; Flags: DWORD; var Credential: PCREDENTIALW): BOOL; stdcall; external CREDAPI name 'CredReadW';
procedure CredFree(Buffer: PVOID); stdcall; external CREDAPI name 'CredFree';

function CredGetMaxPersistType(CredType: DWORD; var MaxCredPersistType: DWORD): Boolean;
const CRED_TYPE_MAXIMUM = 5;
var MaximumPersist: array[0..CRED_TYPE_MAXIMUM-1] of DWORD;
begin
     Result := CredGetSessionTypes(CRED_TYPE_MAXIMUM, PDWORD(@MaximumPersist[0]));
     if Result then MaxCredPersistType := MaximumPersist[CredType] else MaxCredPersistType := 0;
end;

function CredWriteCredentials(const Target, Username, Password: WideString ; CredType: DWORD=CRED_TYPE_DEFAULT): Boolean;
var Credentials: CREDENTIALW;
begin
     if Target<>'' then
     begin
          ZeroMemory(@Credentials, SizeOf(Credentials));
          Credentials.TargetName := PWideChar(Target);
          Credentials.Type_ := CredType;
          Credentials.UserName := PWideChar(Username);
          Credentials.Persist := CRED_PERSIST_LOCAL_MACHINE;
          Credentials.CredentialBlob := LPBYTE(Password);
          Credentials.CredentialBlobSize := 2*(Length(Password));
          Result := CredWriteW(@Credentials, 0);
          ZeroMemory(@Credentials, SizeOf(Credentials));
     end else Result:=true;
end;

function CredReadCredentials(const Target: WideString ; var UserName, Password: WideString ; CredType: DWORD=CRED_TYPE_DEFAULT): bool;
   function WideCharToWideString(Source: PWideChar; SourceLen: Integer): WideString;
   begin if (SourceLen <= 0) then Result := '' else begin SetLength(Result, SourceLen div 2); Move(Source^, Result[1], SourceLen); end; end;
var Credential: PCREDENTIALW;
begin
     Credential := nil; UserName:=''; Password:='';
     if Target<>'' then
     begin
          result:=CredReadW(PWideChar(Target), CredType, 0, Credential);
          if result then
          try
             UserName := Credential.UserName;
             Password := WideCharToWideString(PWideChar(Credential.CredentialBlob), Credential.CredentialBlobSize);
          finally
             CredFree(Credential);
          end;
     end else result:=true;
end;

function CredDeleteCredentials(const Target: WideString ; CredType: DWORD=CRED_TYPE_DEFAULT): Boolean;
begin if Target<>'' then Result:=CredDeleteW(PWideChar(Target), CredType, 0) else result:=true; end;

end.
